#version 130

precision highp float;

uniform sampler2D depthTexture;
uniform sampler2D colorTexture;
uniform mat4 inverseViewProj;
uniform mat4 previousViewProj;
uniform int samples;
uniform vec2 timeDiff;

in vec2 vertexUvs;

out vec4 fragColor;

// Code von http://http.developer.nvidia.com/GPUGems3/gpugems3_ch27.html und fr GLSL angepasst

vec2 getVelocity(void)
{
	float depth = texture(depthTexture, vertexUvs).r;  
	vec4 viewFrag = vec4(vertexUvs.x * 2 - 1, (1 - vertexUvs.y) * 2 - 1, depth, 1);  
	vec4 worldFrag = inverseViewProj*viewFrag;  
	vec4 worldPosition = worldFrag / worldFrag.w;

	vec4 currentPosition = viewFrag;  
	vec4 previousPosition = previousViewProj * worldPosition; 
	previousPosition /= previousPosition.w;
	return (currentPosition - previousPosition).xy;  
}

void main(void)
{	
	vec2 texCoord = vertexUvs;	
	vec2 velocity = vec2(-getVelocity().x, getVelocity().y)/(timeDiff*400.0f);
	
	vec4 color = texture(colorTexture, texCoord);  
	texCoord += velocity;  
	
	for(int i = 1; i < samples; ++i, texCoord += velocity) {  
		vec4 currentColor = texture(colorTexture, texCoord);  
		color += currentColor;
	}  
	
	vec4 finalColor = color/samples;
	
	fragColor = finalColor;
    fragColor.w = 1.0;
}
