#version 130

precision highp float;

uniform vec4 point_light_color;
uniform vec4 direct_light_color;
uniform vec4 ambient_light_color;
uniform vec4 phong_attrib;
uniform sampler2D texture_diff1;
uniform sampler2DShadow shadowMap;

in vec3 normal_view;
in vec4 position_view;
in vec4 vert_color;
in vec2 vert_uvs;
in vec4 point_light_position_view;
//in vec3 direct_light_direction_view;
in vec4 direct_light_position_view;
in vec4 shadowMapTextureCoordinates;

out vec4 fragColor;
out vec4 fake;

void main(void)
{
	float shadow;
	//Shadowmapping teilweise nach ... siehe Shadowmap.hpp
    if (shadowMapTextureCoordinates.w > 1.0) {
		shadow =  textureProj(shadowMap, shadowMapTextureCoordinates);
    } else shadow = 1.0;

	// alles normalisieren ... ist scheinbar besser im frag
	
	// Surface data
	vec3 normal_view_n = normalize(normal_view);
	vec3 position_view_n = position_view.xyz / position_view.w;
	
	// Direct Light
	//vec3 dl_dir = normalize(direct_light_direction_view);
	
	// Direct Light
	vec3 dl_pos = direct_light_position_view.xyz / direct_light_position_view.w;
    
	// calculate the light-direction
    vec3 dl_dir = normalize(dl_pos - position_view_n);
    
	float dl_ndotl = dot(normal_view_n, dl_dir);
	vec4 dl_diffuse =  phong_attrib.y * direct_light_color * max(0.0, dl_ndotl);
    
    vec3 dl_reflection = reflect(dl_dir,normal_view_n)*-1;
    vec4 dl_specular = phong_attrib.z * direct_light_color * max(0.0, pow(dot(normalize(position_view_n)*-1, dl_reflection),phong_attrib.w));
    	
	// Point Light
	vec3 pl1_pos = point_light_position_view.xyz / point_light_position_view.w;
	
	// calculate the light-direction
	float distance = length(pl1_pos - position_view_n);
    vec3 pl1_dir = normalize(pl1_pos - position_view_n);
    
    // falloff: linear - mal 5 sonst ist es zu schwach
	float falloff = (1/distance)*5;
	
	float pl1_ndotl = dot(normal_view_n, pl1_dir);
	vec4 pl1_diffuse =  falloff*phong_attrib.y * point_light_color * max(0.0, pl1_ndotl);
    
    vec3 pl1_reflection = reflect(pl1_dir,normal_view_n)*-1;
    vec4 pl1_specular = falloff*phong_attrib.z * point_light_color * max(0.0, pow(dot(normalize(position_view_n)*-1, pl1_reflection),phong_attrib.w));
            
    // Ambient and Texture
    vec4 tex_color_diff1 = vec4(texture(texture_diff1, vert_uvs));
    vec4 ambient =  phong_attrib.x * ambient_light_color;
    
    fragColor = (ambient + (dl_diffuse*shadow + pl1_diffuse))*(vert_color+tex_color_diff1) + (shadow*dl_specular + pl1_specular);
   
    if(vert_color.xyz.length>0)
   		fragColor.w = vert_color.w;
   	else  	
   		fragColor.w = tex_color_diff1.w;
}
